{{
**************************************************************************
*
*   Serial_Repeater.spin
*   FullDuplexSerial Repeater Driver V1.0
*   October 2014 Marcus Garfunkel
*
**************************************************************************
}}
{
  Serial Repeater for use with FullDuplexSerial Object

Description:

This object runs in its own cog and passes data back and forth between two
FDS ports. The purpose of which causes the propeller to appear transparent
when communicating to peripherals from a host computer. The data from
both the host and the peripheral is buffered in a queue that can be accessed
by the propeller at any time. Since this runs in a separate cog, the Prop
can function as it normally would to send data to the host and peripheral. 


}
   
  fifoSize                = 100

  'Generic Constants
  OFF                     = 0
  ON                      = 1
  YES                     = 0
  NO                      = 1

  ECHO                    = OFF


VAR
  long  cog
  long  Stack[100]

''FDS Variables:
  long  FDS_host
  long  FDS_Perf
  long  DEBUG
  
''Fifo Variables:
  long  fifo1MemAddr[fifoSize]
  long  fifo2MemAddr[fifoSize]
  
OBJ

  FDS      : "fullDuplexSerial4port.spin"
  DIO      : "dataIO4port.spin"
  FIFO1     : "LockQueue1.spin"
  FIFO2     : "LockQueue2.spin"
  'STR      : "ASCII0_STREngine_1.spin"
  
PUB go(Host_fds_port, Peripheral_fds_port)

  stop
  Initialize

  FDS_host := Host_fds_port
  FDS_Perf := Peripheral_fds_port
  DEBUG := FDS_host
  
  cog := cognew(Loop, @Stack) + 1  'start assembly cog
  return cog

PUB stop
                                                                                
  if cog                                           'Is cog non-zero?
    FIFO1.destroy
    FIFO2.destroy 
    cogstop(cog~ - 1)                              'Yes, stop the cog and then make value zero

PRI Initialize | returnVal, index    

  returnVal := FIFO1.Initialize(@fifo1MemAddr, fifoSize)
  fds.str(DEBUG, String("fifo1 (internal UI generated) initiated. Semaphore used: "))
  dio.dec(DEBUG, returnVal)
  fds.tx(DEBUG, 13)

  returnVal := FIFO2.Initialize(@fifo2MemAddr, fifoSize)
  fds.str(DEBUG, String("fifo2 (internal UI generated) initiated. Semaphore used: "))
  dio.dec(DEBUG, returnVal)
  fds.tx(DEBUG, 13)


PRI Loop | returnVal

  repeat

    'Check to see if there is any incomming character from Host
    if (returnVal := fds.rxCheck(FDS_host)) <> -1 
      'add to Host data buffer and pass along to Peripheral   
      FIFO1.push(returnVal)
      fds.tx(FDS_Perf, returnVal)
      if ECHO
        fds.tx(FDS_host, returnVal)  

    'Check to see if there is any incomming character from Peripheral
    if (returnVal := fds.rxCheck(FDS_Perf)) <> -1 
      'add to Periferal data buffer and pass along to Host   
      FIFO2.push(returnVal)
      fds.tx(FDS_host, returnVal)   

DAT
